"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var esprima = require('esprima');
var escodegen = require('escodegen');
exports.convertStyle = function (styleString) {
    var styleObj = JSON.parse(styleString.replace(/'/g, "\""));
    return Object.keys(styleObj)
        .map(function (key) { return kebabCase(key) + ": " + styleObj[key]; })
        .join('; ');
};
var kebabCase = function (string) {
    return lodash_1.words(("" + string).replace(/['\u2019]/g, '')).reduce(function (result, word, index) { return result + (index ? '-' : '') + word.toLowerCase(); }, '');
};
exports.file = kebabCase;
var Event_1 = require("./Event");
function isAngularComponent(type) {
    return type != 'row' && type != 'col';
}
exports.isAngularComponent = isAngularComponent;
function isComponent(type) {
    return (!/^(array|object|string|number|integer|boolean)$/.test(type) &&
        !/-(item|level|parameter|option|form-field|column|header|footer|marker)$/.test(type));
}
exports.isComponent = isComponent;
function componentTypes(layouts, pages) {
    var componentTypes = new Set();
    function collect(component) {
        var type = component.type;
        if (type && isAngularComponent(type) && isComponent(type)) {
            componentTypes.add(type);
        }
        [
            'components',
            'columns',
            'items',
            'headerTemplate',
            'editTemplate',
            'filterTemplate',
            'template',
            'footerTemplate'
        ].forEach(function (name) {
            var children = component[name];
            if (Array.isArray(children)) {
                children.forEach(collect);
            }
            else if (typeof children == 'string') {
                var types = inferComponentsFromTemplate(children);
                types.forEach(function (type) { return componentTypes.add(type); });
            }
        });
    }
    layouts.forEach(function (layout) { return layout.components.forEach(collect); });
    pages.forEach(function (page) { return page.components.forEach(collect); });
    return Array.from(componentTypes).concat('shared', 'notification', 'dialog');
}
exports.componentTypes = componentTypes;
function inferComponentsFromTemplate(template) {
    var result = new Set();
    var componentRe = /<rz-([\w-]+)/g;
    var match = null;
    do {
        match = componentRe.exec(template);
        if (match) {
            var component = match[1];
            if (isComponent(component)) {
                result.add(component);
            }
        }
    } while (match);
    return Array.from(result);
}
exports.inferComponentsFromTemplate = inferComponentsFromTemplate;
function odataPrefix(dataSource) {
    var basePath = dataSource.basePath;
    return basePath.substring(basePath.indexOf('odata/'));
}
exports.odataPrefix = odataPrefix;
function constant(value, suffix) {
    if (suffix) {
        return (lodash_1.snakeCase(value) + "_" + lodash_1.snakeCase(suffix)).toUpperCase();
    }
    return value.toUpperCase();
}
exports.constant = constant;
function validModel(model) {
    return (Object.keys(model.properties).length == 0 ||
        model.properties.filter(function (property) { return property.key && property.type != 'object'; }).length > 0);
}
exports.validModel = validModel;
function hasValue(type) {
    return /^(file-input|rating|dropdown-datagrid|datepicker|dropdown|listbox|numeric|textbox|textarea|autocomplete|selectbar|password|checkbox|mask|slider|radiobuttonlist|checkboxlist)$/.test(type);
}
exports.hasValue = hasValue;
function stringify(value) {
    return JSON.stringify(value).replace(/"/g, "'");
}
exports.stringify = stringify;
function renderRefType(component) {
    var componentRefType = componentType(component.type);
    if (componentRefType == 'Grid' ||
        componentRefType == 'TemplateForm' ||
        componentRefType == 'DataList' ||
        componentRefType == 'ListBox') {
        return component.itemType
            ? "Radzen" + componentRefType + "<" + component.itemType + ">"
            : "dynamic";
    }
    else if (componentRefType == 'Numeric') {
        return "dynamic";
    }
    return "Radzen" + componentRefType;
}
exports.renderRefType = renderRefType;
function eventHandlerType(event) {
    return event.name == 'rowRender' || event.name == 'cellRender'
        ? 'void'
        : 'System.Threading.Tasks.Task';
}
exports.eventHandlerType = eventHandlerType;
function eventArgsType(component, event) {
    var componentRefType = componentType(component.type);
    if (componentRefType == 'TemplateForm' &&
        event.name == 'submit' &&
        component.itemType) {
        return component.itemType;
    }
    if (componentRefType == 'Grid' && event.name == 'loadData') {
        return "LoadDataArgs";
    }
    if (componentRefType == 'SplitButton' && event.name == 'click') {
        return "RadzenSplitButtonItem";
    }
    if (componentRefType == 'Grid' && event.name == 'rowRender') {
        return "RowRenderEventArgs<" + (component.itemType || 'dynamic') + ">";
    }
    if (componentRefType == 'Grid' && event.name == 'cellRender') {
        return "CellRenderEventArgs<" + (component.itemType || 'dynamic') + ">";
    }
    if (componentRefType == 'Tree') {
        switch (event.name) {
            case 'expand':
                return 'TreeExpandEventArgs';
            default:
                return 'TreeEventArgs';
        }
    }
    if (componentRefType == 'Grid' &&
        (event.name == 'rowSelect' || event.name == 'rowExpand') &&
        component.itemType) {
        return component.itemType;
    }
    if (componentRefType == 'Button' && event.name == 'click') {
        return "MouseEventArgs";
    }
    if (componentRefType == 'DatePicker' && event.name == 'change') {
        return "DateTime?";
    }
    if ((componentRefType == 'TextBox' ||
        componentRefType == 'TextArea' ||
        componentRefType == 'FileInput') &&
        event.name == 'change') {
        return "string";
    }
    if (componentRefType == 'Numeric' && event.name == 'change') {
        var valueTypeProperty = component.properties.find(function (property) { return property.name == 'valueType'; });
        return valueTypeProperty ? valueTypeProperty.value : "decimal?";
    }
    if (componentRefType == 'CheckBox' && event.name == 'change') {
        var valueTypeProperty = component.properties.find(function (property) { return property.name == 'valueType'; });
        return valueTypeProperty ? valueTypeProperty.value : "bool?";
    }
    if (componentRefType == 'Login' && event.name == 'register') {
        return "";
    }
    if (event.actions && event.actions[0] && event.actions[0].argType) {
        return event.actions[0].argType;
    }
    return 'dynamic';
}
exports.eventArgsType = eventArgsType;
exports.localizable = [
    'text',
    'title',
    'placeholder',
    'tooltip',
    'loginText',
    'registerText',
    'registerMessageText',
    'resetPasswordText',
    'userText',
    'userRequired',
    'passwordText',
    'passwordRequired',
    'emptyText',
    'defaultRowText',
    'chooseText',
    'confirmDeleteTitle',
    'confirmDeleteText',
    'confirmDeleteOkText',
    'confirmDeleteCancelText',
    'applyFilterText',
    'clearFilterText',
    'minText',
    'maxText',
    'defaultItemText',
    'uploadText',
    'cancelText',
    'selectedItemsText'
];
function localizationPropertyKey(component, propertyName) {
    if (component.parent) {
        var parentName = component.parent.name ? component.parent.name + "." : "";
        if (component.type == 'grid-column') {
            var propertyProperty = component.properties.find(function (cp) { return cp.name == 'property'; });
            return "" + parentName + (propertyProperty ? propertyProperty.value : '') + "." + lodash_1.upperFirst(propertyName);
        }
        else if (component.type == 'panelmenu-item') {
            var pathProperty = component.properties.find(function (cp) { return cp.name == 'path'; });
            var textProperty = component.properties.find(function (cp) { return cp.name == 'text'; });
            return "" + parentName + (pathProperty
                ? pathProperty.value
                : textProperty
                    ? lodash_1.camelCase(textProperty.value)
                    : '') + "." + lodash_1.upperFirst(propertyName);
        }
        else if (component.type == 'profilemenu-item') {
            var textProperty = component.properties.find(function (cp) { return cp.name == 'text'; });
            return "" + parentName + (textProperty ? lodash_1.camelCase(textProperty.value) : '') + "." + lodash_1.upperFirst(propertyName);
        }
    }
    else {
        return component.name + "." + lodash_1.upperFirst(propertyName);
    }
}
exports.localizationPropertyKey = localizationPropertyKey;
function resolveGenericArgument(type, properties, events, designTime, parentValue) {
    var argumentName = '';
    switch (type) {
        case 'grid':
        case 'datalist':
        case 'template-form':
            argumentName = 'itemType';
            break;
        case 'numeric':
        case 'dropdown':
        case 'dropdown-datagrid':
        case 'datepicker':
        case 'listbox':
        case 'checkbox':
        case 'checkboxlist':
        case 'checkboxlist-item':
        case 'radiobuttonlist':
        case 'radiobuttonlist-item':
        case 'selectbar':
        case 'selectbar-item':
        case 'file-input':
        case 'languagepicker':
            argumentName = 'valueType';
            break;
    }
    if (argumentName == 'valueType') {
        var valueTypeProperty = properties.find(function (property) { return property.name == 'valueType'; });
        var valueProperty = properties.find(function (property) {
            return property.name == 'value' &&
                property.value !== undefined &&
                (/\${/.test(property.value) || /\@/.test(property.value));
        }) ||
            ((type == 'checkboxlist-item' || type == 'radiobuttonlist-item') &&
                parentValue &&
                (/\${/.test(parentValue) || /\@/.test(parentValue)));
        var changeEvent = Array.isArray(events) && events.length
            ? events.find(function (e) { return e.name == 'change'; })
            : null;
        if ((!valueProperty || designTime) && !valueTypeProperty) {
            if (type == 'checkbox' && changeEvent) {
                var checkboxTriStateProperty = properties.find(function (p) { return p.name == 'triState'; });
                properties.push({
                    name: 'valueType',
                    value: checkboxTriStateProperty ? 'bool?' : 'bool'
                });
            }
            else if (type == 'numeric' && changeEvent) {
                properties.push({ name: 'valueType', value: 'decimal?' });
            }
            else {
                // Component doesn't have valueType set - make it dynamic
                properties.push({ name: 'valueType', value: 'dynamic' });
            }
        }
        if (!designTime &&
            valueProperty &&
            valueTypeProperty &&
            !((type == 'checkbox' || type == 'numeric') && changeEvent)) {
            // Component has both valueType and value set - remove valueType
            var valueTypeIndex = properties.indexOf(valueTypeProperty);
            properties.splice(valueTypeIndex, 1);
        }
    }
    else if (argumentName == 'itemType') {
        var itemTypeProperty = properties.find(function (property) { return property.name == 'itemType'; });
        var dataProperty = properties.find(function (property) { return property.name == 'data'; });
        if (!itemTypeProperty && !dataProperty) {
            // Component doesn't have valueType set - make it dynamic
            properties.push({ name: 'itemType', value: 'dynamic' });
        }
    }
    else {
        var valueTypeIndex = properties.findIndex(function (property) { return property.name == 'valueType'; });
        if (valueTypeIndex >= 0) {
            properties.splice(valueTypeIndex, 1);
        }
    }
}
function renderBlazorComponent(designTime, component, application, options) {
    if (designTime === void 0) { designTime = false; }
    var parent = component.parent, type = component.type, name = component.name, properties = component.properties, events = component.events, attributes = component.attributes, components = component.components, template = component.template, editTemplate = component.editTemplate, filterTemplate = component.filterTemplate, headerTemplate = component.headerTemplate, footerTemplate = component.footerTemplate, content = component.content;
    var itemType;
    var html = !isAngularComponent(type);
    var ref = name && (type == 'grid' || type == 'body' || type == 'sidebar')
        ? " @ref=\"" + componentRefName(name) + "\""
        : '';
    var input = hasValue(type);
    if ((input && Array.isArray(properties)) || type == 'content-container') {
        properties.push({ name: 'name', value: name, bound: false });
    }
    var parentValue = parent ? parent.value : null;
    resolveGenericArgument(type, properties, events, designTime, parentValue);
    var filteredProperties = Array.isArray(properties)
        ? properties.filter(function (property) {
            var value = property.value, name = property.name;
            if (type == 'custom-component' && name == 'page') {
                return false;
            }
            if (type == 'link' && name == 'parameters') {
                return false;
            }
            if (name == 'visible' && value == true) {
                return false;
            }
            if (name == 'valueFormat') {
                return false;
            }
            if (input && name == 'value' && designTime) {
                return false;
            }
            if (component.type == 'dropdown-datagrid' &&
                (name == 'selectedValue' || name == 'itemType')) {
                return false;
            }
            if ((component.type == 'tabs-item' || component.type == 'steps-item') &&
                name == 'selected' &&
                !designTime) {
                return false;
            }
            return (value != null &&
                value !== '' &&
                (!Array.isArray(value) || value.length > 0));
        })
        : [];
    var props = filteredProperties.length > 0
        ? ' ' +
            filteredProperties
                .map(function (property) {
                var name = property.name, value = property.value, bound = property.bound;
                var valueType = typeof value;
                if (valueType == 'string') {
                    value = value.replace(/parameters\./g, '');
                }
                if (Array.isArray(value)) {
                    value = stringify(value);
                    bound = true;
                }
                if (!bound && value && value.includes && value.includes('${')) {
                    bound = true;
                }
                if (name == 'path' && type == 'image') {
                    if (bound) {
                        if (options.itemType && options.itemType != 'dynamic') {
                            value = interpolateAsBlazorAttribute([], value.replace('data.', lodash_1.camelCase(options.itemType) + "."));
                        }
                    }
                    if (value && value.indexOf('/assets') == 0) {
                        value = value.substring(1);
                    }
                    return lodash_1.upperFirst(name) + "=\"" + value + "\"";
                }
                if (name == 'style') {
                    if (!value) {
                        return '';
                    }
                    return "style=\"" + exports.convertStyle(value) + "\"";
                }
                else {
                    if (!html) {
                        if (bound) {
                            if (options.itemType && options.itemType != 'dynamic') {
                                value = interpolateAsBlazorAttribute([], value.replace('data.', lodash_1.camelCase(options.itemType) + "."));
                            }
                        }
                        if (name == 'value' && input) {
                            if (bound) {
                                return "@bind-Value=\"" + value + "\"";
                            }
                            else {
                                return "Value=\"" + value + "\"";
                            }
                        }
                        if (name == 'value' && type == 'ssrsviewer-parameter') {
                            if (bound) {
                                return "Value=\"@($\"{" + value.replace('@', '') + "}\")\"";
                            }
                            else {
                                return "Value=\"" + value + "\"";
                            }
                        }
                        if (name == 'text' && type == 'label') {
                            if (bound) {
                                return "Text=\"@($\"{" + value.replace('@', '') + "}\")\"";
                            }
                            else {
                                if (!designTime &&
                                    application &&
                                    application.localization &&
                                    application.localization.cultures.length > 1 &&
                                    exports.localizable.indexOf(name) != -1) {
                                    return lodash_1.upperFirst(name) + "=\"@L[\"" + localizationPropertyKey(component, name) + "\"]\"";
                                }
                                else {
                                    return "Text=\"" + value + "\"";
                                }
                            }
                        }
                        if (name == 'path' && type == 'link') {
                            var pathParameters = (properties || []).find(function (a) { return a.name == 'parameters'; });
                            var pathParametersValues = (pathParameters ? JSON.parse(pathParameters.value.replace(/'/g, '"')) : []).map(function (p) { return p.value; });
                            return lodash_1.upperFirst(name) + "=\"" + value + (pathParametersValues.length ? '/' : '') + pathParametersValues.join('/') + "\"";
                        }
                        if (name == 'mode' && type == 'progressbar') {
                            return lodash_1.upperFirst(name) + "=\"ProgressBarMode." + lodash_1.upperFirst(value) + "\"";
                        }
                        if (name == 'size' && type == 'button') {
                            return lodash_1.upperFirst(name) + "=\"ButtonSize." + (value == 'md' ? 'Medium' : 'Small') + "\"";
                        }
                        if (name == 'buttonType' && type == 'button') {
                            return lodash_1.upperFirst(name) + "=\"ButtonType." + lodash_1.upperFirst(value) + "\"";
                        }
                        if (name == 'buttonStyle' && type == 'button') {
                            return lodash_1.upperFirst(name) + "=\"ButtonStyle." + lodash_1.upperFirst(value) + "\"";
                        }
                        if (name == 'filterCaseSensitivity') {
                            return lodash_1.upperFirst(name) + "=\"FilterCaseSensitivity." + lodash_1.upperFirst(value) + "\"";
                        }
                        if (name == 'textAlign' && type == 'grid-column') {
                            return lodash_1.upperFirst(name) + "=\"TextAlign." + lodash_1.upperFirst(value) + "\"";
                        }
                        if (name == 'orientation') {
                            return lodash_1.upperFirst(name) + "=\"Orientation." + lodash_1.upperFirst(value) + "\"";
                        }
                        if (name == 'itemType' && type != 'dropdown-datagrid') {
                            itemType = value;
                            var customTItem = (attributes || []).find(function (a) { return a.name == 'TItem'; });
                            if (customTItem) {
                                itemType = customTItem.value;
                            }
                            return !customTItem ? "TItem=\"" + value + "\"" : '';
                        }
                        if (name == 'valueType') {
                            return designTime || !(Array.isArray(attributes) && attributes.find(function (a) { return a.name == 'TValue'; })) ? "TValue=\"" + value + "\"" : '';
                        }
                        if (input && name == 'name') {
                            return lodash_1.upperFirst(name) + "=\"" + lodash_1.upperFirst(value) + "\"";
                        }
                        if (type.indexOf('validator') != -1 && name == 'component') {
                            return lodash_1.upperFirst(name) + "=\"" + lodash_1.upperFirst(value) + "\"";
                        }
                        if (!designTime &&
                            application &&
                            application.localization &&
                            application.localization.cultures.length > 1 &&
                            exports.localizable.indexOf(name) != -1 &&
                            !bound) {
                            return lodash_1.upperFirst(name) + "=\"@L[\"" + localizationPropertyKey(component, name) + "\"]\"";
                        }
                        return lodash_1.upperFirst(name) + "=\"" + value + "\"";
                    }
                    else {
                        return name + "=\"" + value + "\"";
                    }
                }
            })
                .join(' ')
        : '';
    var evs = Array.isArray(events) && events.length
        ? ' ' +
            events
                .map(function (event) {
                if (component.inTemplate) {
                    return lodash_1.upperFirst(event.name) + "=\"@((args) =>" + lodash_1.upperFirst(name) + lodash_1.upperFirst(event.name) + "(args, " + lodash_1.camelCase(options.itemType || 'data') + "))\"";
                }
                else {
                    return lodash_1.upperFirst(event.name) + "=\"@" + pascalCase(event.value) + "\"";
                }
            })
                .join(' ')
        : '';
    var pageProperty = (properties || []).find(function (p) { return p.name == 'page'; });
    var tagName = type
        ? html
            ? 'div'
            : type == 'custom-component'
                ? (pageProperty && pageProperty.value ? pageProperty.value : '').split(' ').join('')
                : "Radzen" + lodash_1.upperFirst(componentType(type))
        : '';
    var context = function () {
        if (tagName == 'RadzenGridColumn') {
            return " TItem=\"" + (options.itemType || 'dynamic') + "\"";
        }
        else {
            return '';
        }
    };
    var attrs = !designTime && Array.isArray(attributes) && attributes.length
        ? ' ' +
            attributes
                .filter(function (a) { return a.name && a.value; })
                .map(function (a) { return (a.name + "=\"" + a.value + "\"").replace('${', '@').replace('}', ''); })
                .join(' ')
        : '';
    var parts = type
        ? ["<" + tagName + ref + context() + props + evs + attrs + ">"]
        : [];
    if (html) {
        if (Array.isArray(components) && components.length) {
            var componentContent = components
                .map(function (component) {
                return indent(renderBlazorComponent(designTime, component, application, options), 2);
            })
                .join('\n');
            parts.push(componentContent);
        }
    }
    else if (type == 'grid') {
        if (Array.isArray(components) && components.length) {
            options.itemType = itemType;
            var componentContent = indent("<Columns>", 2) +
                '\n' +
                components
                    .map(function (component) {
                    return indent(renderBlazorComponent(designTime, component, application, options), 4);
                })
                    .join('\n') +
                ("\n" + indent('</Columns>', 2));
            parts.push(componentContent);
        }
    }
    else if (type == 'form') {
        if (Array.isArray(components) && components.length) {
            var componentContent = indent("<Fields>", 2) +
                '\n' +
                components
                    .map(function (component) {
                    return indent(renderBlazorComponent(designTime, component, application, options), 4);
                })
                    .join('\n') +
                ("\n" + indent('</Fields>', 2));
            parts.push(componentContent);
        }
    }
    else if (type == 'tabs') {
        if (Array.isArray(components) && components.length) {
            var componentContent = indent("<Tabs>", 2) +
                '\n' +
                components
                    .map(function (component) {
                    return indent(renderBlazorComponent(designTime, component, application, options), 4);
                })
                    .join('\n') +
                ("\n" + indent('</Tabs>', 2));
            parts.push(componentContent);
        }
    }
    else if (type == 'steps') {
        if (Array.isArray(components) && components.length) {
            var componentContent = indent("<Steps>", 2) +
                '\n' +
                components
                    .map(function (component) {
                    return indent(renderBlazorComponent(designTime, component, application, options), 4);
                })
                    .join('\n') +
                ("\n" + indent('</Steps>', 2));
            parts.push(componentContent);
        }
    }
    else if (type == 'accordion' ||
        type == 'selectbar' ||
        type == 'checkboxlist' ||
        type == 'radiobuttonlist') {
        if (Array.isArray(components) && components.length) {
            var componentContent = indent("<Items>", 2) +
                '\n' +
                components
                    .map(function (component) {
                    return indent(renderBlazorComponent(designTime, component, application, options), 4);
                })
                    .join('\n') +
                ("\n" + indent('</Items>', 2));
            parts.push(componentContent);
        }
    }
    else if (type == 'ssrsviewer') {
        if (Array.isArray(components) && components.length) {
            var componentContent = indent("<Parameters>", 2) +
                '\n' +
                components
                    .map(function (component) {
                    return indent(renderBlazorComponent(designTime, component, application, options), 4);
                })
                    .join('\n') +
                ("\n" + indent('</Parameters>', 2));
            parts.push(componentContent);
        }
    }
    else {
        if (Array.isArray(components) && components.length) {
            var componentContent = indent("<ChildContent>", 2) +
                '\n' +
                components
                    .map(function (component) {
                    return indent(renderBlazorComponent(designTime, component, application, options), 4);
                })
                    .join('\n') +
                ("\n" + indent('</ChildContent>', 2));
            parts.push(componentContent);
        }
    }
    if (Array.isArray(template) && template.length) {
        var templateContent = indent(type == 'profilemenu'
            ? "<Template>"
            : "<Template Context=\"" + lodash_1.camelCase(options.itemType && options.itemType != 'dynamic' ? options.itemType : 'data') + "\">", 2) +
            '\n' +
            template
                .map(function (component) {
                return indent(renderBlazorComponent(designTime, component, application, options), 4);
            })
                .join('\n') +
            ("\n" + indent('</Template>', 2));
        parts.push(templateContent);
    }
    if (Array.isArray(editTemplate) && editTemplate.length) {
        var templateContent = indent("<EditTemplate Context=\"" + lodash_1.camelCase(options.itemType && options.itemType != 'dynamic' ? options.itemType : 'data') + "\">", 2) +
            '\n' +
            editTemplate
                .map(function (component) {
                return indent(renderBlazorComponent(designTime, component, application, options), 4);
            })
                .join('\n') +
            ("\n" + indent('</EditTemplate>', 2));
        parts.push(templateContent);
    }
    if (Array.isArray(filterTemplate) && filterTemplate.length) {
        var templateContent = indent("<FilterTemplate>", 2) +
            '\n' +
            filterTemplate
                .map(function (component) {
                return indent(renderBlazorComponent(designTime, component, application, options), 4);
            })
                .join('\n') +
            ("\n" + indent('</FilterTemplate>', 2));
        parts.push(templateContent);
    }
    if (Array.isArray(headerTemplate) && headerTemplate.length) {
        var headerTemplateContent = indent("<HeaderTemplate>", 2) +
            '\n' +
            headerTemplate
                .map(function (component) {
                return indent(renderBlazorComponent(designTime, component, application, options), 4);
            })
                .join('\n') +
            ("\n" + indent("</HeaderTemplate>", 2));
        parts.push(headerTemplateContent);
    }
    if (content) {
        parts.push(indent(content, 2));
    }
    if (Array.isArray(footerTemplate) && footerTemplate.length) {
        var footerTemplateContent = indent("<FooterTemplate>", 2) +
            '\n' +
            footerTemplate
                .map(function (component) {
                return indent(renderBlazorComponent(designTime, component, application, options), 4);
            })
                .join('\n') +
            ("\n" + indent("</FooterTemplate>", 2));
        parts.push(footerTemplateContent);
    }
    if (tagName) {
        if (tagName == 'RadzenContentContainer') {
            parts.push("@Body");
        }
        parts.push("</" + tagName + ">");
    }
    return parts.join('\n');
}
exports.renderBlazorComponent = renderBlazorComponent;
function type(value) {
    if (primitive(value)) {
        return value == 'integer' ? 'number' : value;
    }
    if (!isNaN(value.charAt(0))) {
        value = "C" + value;
    }
    value = lodash_1.camelCase(unwrapType(value));
    return value.charAt(0).toUpperCase() + value.substring(1);
}
exports.type = type;
function componentType(value) {
    if (value == 'progressbar') {
        return 'ProgressBar';
    }
    if (value == 'profilemenu') {
        return 'ProfileMenu';
    }
    if (value == 'languagepicker') {
        return 'LanguagePicker';
    }
    if (value == 'panelmenu') {
        return 'PanelMenu';
    }
    if (value == 'panelmenu-item') {
        return 'PanelMenuItem';
    }
    if (value == 'menu-item') {
        return 'MenuItem';
    }
    if (value == 'profilemenu-item') {
        return 'ProfileMenuItem';
    }
    if (value == 'textbox') {
        return 'TextBox';
    }
    if (value == 'listbox') {
        return 'ListBox';
    }
    if (value == 'textarea') {
        return 'TextArea';
    }
    if (value == 'radiobuttonlist') {
        return 'RadioButtonList';
    }
    if (value == 'radiobuttonlist-item') {
        return 'RadioButtonListItem';
    }
    if (value == 'checkboxlist') {
        return 'CheckBoxList';
    }
    if (value == 'checkboxlist-item') {
        return 'CheckBoxListItem';
    }
    if (value == 'dropdown') {
        return 'DropDown';
    }
    if (value == 'dropdown-datagrid') {
        return 'DropDownDataGrid';
    }
    if (value == 'splitbutton') {
        return 'SplitButton';
    }
    if (value == 'splitbutton-item') {
        return 'SplitButtonItem';
    }
    if (value == 'ssrsviewer') {
        return 'SSRSViewer';
    }
    if (value == 'ssrsviewer-parameter') {
        return 'SSRSViewerParameter';
    }
    if (value == 'datalist') {
        return 'DataList';
    }
    if (value == 'picklist') {
        return 'PickList';
    }
    if (value == 'selectbar') {
        return 'SelectBar';
    }
    if (value == 'selectbar-item') {
        return 'SelectBarItem';
    }
    if (value == 'checkbox') {
        return 'CheckBox';
    }
    if (value == 'datepicker') {
        return 'DatePicker';
    }
    if (value == 'treegrid') {
        return 'TreeGrid';
    }
    if (value == 'autocomplete') {
        return 'AutoComplete';
    }
    if (value == 'tabstrip') {
        return 'TabStrip';
    }
    if (value == 'googlemap') {
        return 'GoogleMap';
    }
    return type(value);
}
exports.componentType = componentType;
function pascalCase(value) {
    return lodash_1.upperFirst(lodash_1.camelCase(value));
}
exports.pascalCase = pascalCase;
function canEditProperty(property) {
    var format = property.format;
    return format != 'uuid' && format != 'time';
}
exports.canEditProperty = canEditProperty;
function dotnetType(property, lang) {
    if (lang === void 0) { lang = 'cs'; }
    var type = property.type, format = property.format, nullable = property.nullable;
    var result = null;
    if (lang == 'cs') {
        switch (type) {
            case 'integer':
                if (format == 'int16') {
                    result = 'Int16';
                }
                else if (format == 'int8') {
                    result = 'Byte';
                }
                else if (format == 'int64') {
                    result = 'Int64';
                }
                else {
                    result = 'int';
                }
                break;
            case 'string':
                if (format == 'date-time' || format == 'date') {
                    result = 'DateTime';
                }
                else if (format == 'time') {
                    result = 'TimeSpan';
                }
                else if (format == 'date-time-offset') {
                    result = 'DateTimeOffset';
                }
                else if (format == 'binary') {
                    result = 'Byte[]';
                }
                else if (format == 'uuid') {
                    result = 'Guid';
                }
                else {
                    result = 'string';
                }
                break;
            case 'number':
                if (format == 'double') {
                    result = 'double';
                }
                else if (format == 'decimal') {
                    result = 'decimal';
                }
                else {
                    result = 'float';
                }
                break;
            case 'boolean':
                result = 'bool';
                break;
            case 'object':
                result = 'object';
                break;
            default:
                throwCodegenError(null, null, null, "Unsupported type " + type);
        }
        if (nullable &&
            result != 'object' &&
            result != 'string' &&
            result != 'Byte[]') {
            result = result + '?';
        }
    }
    else if (lang == 'vb') {
        switch (type) {
            case 'integer':
                if (format == 'int16') {
                    result = 'Int16';
                }
                else if (format == 'int8') {
                    result = 'Byte';
                }
                else if (format == 'int64') {
                    result = 'Int64';
                }
                else {
                    result = 'Integer';
                }
                break;
            case 'string':
                if (format == 'date-time') {
                    result = 'DateTime';
                }
                else if (format == 'time') {
                    result = 'TimeSpan';
                }
                else if (format == 'date-time-offset') {
                    result = 'DateTimeOffset';
                }
                else if (format == 'binary') {
                    result = 'Byte()';
                }
                else if (format == 'uuid') {
                    result = 'Guid';
                }
                else {
                    result = 'String';
                }
                break;
            case 'number':
                if (format == 'double') {
                    result = 'Double';
                }
                else if (format == 'decimal') {
                    result = 'Decimal';
                }
                else {
                    result = 'Single';
                }
                break;
            case 'boolean':
                result = 'Boolean';
                break;
            case 'object':
                result = 'Object';
                break;
            default:
                throwCodegenError(null, null, null, "Unsupported type " + type);
        }
        if (nullable &&
            result != 'Object' &&
            result != 'String' &&
            result != 'Byte()') {
            result = "Nullable(Of " + result + ")";
        }
    }
    return result;
}
exports.dotnetType = dotnetType;
function isValidIdentifier(name) {
    return /^(?!(?:event|sealed|object|do|if|in|for|let|new|try|var|case|else|enum|eval|false|null|this|true|void|with|break|catch|class|const|super|throw|while|yield|delete|export|import|public|return|static|switch|typeof|default|extends|finally|namespace|package|private|continue|debugger|function|arguments|interface|protected|implements|instanceof)$)[$A-Z\_a-z\xaa\xb5\xba\xc0-\xd6\xd8-\xf6\xf8-\u02c1\u02c6-\u02d1\u02e0-\u02e4\u02ec\u02ee\u0370-\u0374\u0376\u0377\u037a-\u037d\u0386\u0388-\u038a\u038c\u038e-\u03a1\u03a3-\u03f5\u03f7-\u0481\u048a-\u0527\u0531-\u0556\u0559\u0561-\u0587\u05d0-\u05ea\u05f0-\u05f2\u0620-\u064a\u066e\u066f\u0671-\u06d3\u06d5\u06e5\u06e6\u06ee\u06ef\u06fa-\u06fc\u06ff\u0710\u0712-\u072f\u074d-\u07a5\u07b1\u07ca-\u07ea\u07f4\u07f5\u07fa\u0800-\u0815\u081a\u0824\u0828\u0840-\u0858\u08a0\u08a2-\u08ac\u0904-\u0939\u093d\u0950\u0958-\u0961\u0971-\u0977\u0979-\u097f\u0985-\u098c\u098f\u0990\u0993-\u09a8\u09aa-\u09b0\u09b2\u09b6-\u09b9\u09bd\u09ce\u09dc\u09dd\u09df-\u09e1\u09f0\u09f1\u0a05-\u0a0a\u0a0f\u0a10\u0a13-\u0a28\u0a2a-\u0a30\u0a32\u0a33\u0a35\u0a36\u0a38\u0a39\u0a59-\u0a5c\u0a5e\u0a72-\u0a74\u0a85-\u0a8d\u0a8f-\u0a91\u0a93-\u0aa8\u0aaa-\u0ab0\u0ab2\u0ab3\u0ab5-\u0ab9\u0abd\u0ad0\u0ae0\u0ae1\u0b05-\u0b0c\u0b0f\u0b10\u0b13-\u0b28\u0b2a-\u0b30\u0b32\u0b33\u0b35-\u0b39\u0b3d\u0b5c\u0b5d\u0b5f-\u0b61\u0b71\u0b83\u0b85-\u0b8a\u0b8e-\u0b90\u0b92-\u0b95\u0b99\u0b9a\u0b9c\u0b9e\u0b9f\u0ba3\u0ba4\u0ba8-\u0baa\u0bae-\u0bb9\u0bd0\u0c05-\u0c0c\u0c0e-\u0c10\u0c12-\u0c28\u0c2a-\u0c33\u0c35-\u0c39\u0c3d\u0c58\u0c59\u0c60\u0c61\u0c85-\u0c8c\u0c8e-\u0c90\u0c92-\u0ca8\u0caa-\u0cb3\u0cb5-\u0cb9\u0cbd\u0cde\u0ce0\u0ce1\u0cf1\u0cf2\u0d05-\u0d0c\u0d0e-\u0d10\u0d12-\u0d3a\u0d3d\u0d4e\u0d60\u0d61\u0d7a-\u0d7f\u0d85-\u0d96\u0d9a-\u0db1\u0db3-\u0dbb\u0dbd\u0dc0-\u0dc6\u0e01-\u0e30\u0e32\u0e33\u0e40-\u0e46\u0e81\u0e82\u0e84\u0e87\u0e88\u0e8a\u0e8d\u0e94-\u0e97\u0e99-\u0e9f\u0ea1-\u0ea3\u0ea5\u0ea7\u0eaa\u0eab\u0ead-\u0eb0\u0eb2\u0eb3\u0ebd\u0ec0-\u0ec4\u0ec6\u0edc-\u0edf\u0f00\u0f40-\u0f47\u0f49-\u0f6c\u0f88-\u0f8c\u1000-\u102a\u103f\u1050-\u1055\u105a-\u105d\u1061\u1065\u1066\u106e-\u1070\u1075-\u1081\u108e\u10a0-\u10c5\u10c7\u10cd\u10d0-\u10fa\u10fc-\u1248\u124a-\u124d\u1250-\u1256\u1258\u125a-\u125d\u1260-\u1288\u128a-\u128d\u1290-\u12b0\u12b2-\u12b5\u12b8-\u12be\u12c0\u12c2-\u12c5\u12c8-\u12d6\u12d8-\u1310\u1312-\u1315\u1318-\u135a\u1380-\u138f\u13a0-\u13f4\u1401-\u166c\u166f-\u167f\u1681-\u169a\u16a0-\u16ea\u16ee-\u16f0\u1700-\u170c\u170e-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176c\u176e-\u1770\u1780-\u17b3\u17d7\u17dc\u1820-\u1877\u1880-\u18a8\u18aa\u18b0-\u18f5\u1900-\u191c\u1950-\u196d\u1970-\u1974\u1980-\u19ab\u19c1-\u19c7\u1a00-\u1a16\u1a20-\u1a54\u1aa7\u1b05-\u1b33\u1b45-\u1b4b\u1b83-\u1ba0\u1bae\u1baf\u1bba-\u1be5\u1c00-\u1c23\u1c4d-\u1c4f\u1c5a-\u1c7d\u1ce9-\u1cec\u1cee-\u1cf1\u1cf5\u1cf6\u1d00-\u1dbf\u1e00-\u1f15\u1f18-\u1f1d\u1f20-\u1f45\u1f48-\u1f4d\u1f50-\u1f57\u1f59\u1f5b\u1f5d\u1f5f-\u1f7d\u1f80-\u1fb4\u1fb6-\u1fbc\u1fbe\u1fc2-\u1fc4\u1fc6-\u1fcc\u1fd0-\u1fd3\u1fd6-\u1fdb\u1fe0-\u1fec\u1ff2-\u1ff4\u1ff6-\u1ffc\u2071\u207f\u2090-\u209c\u2102\u2107\u210a-\u2113\u2115\u2119-\u211d\u2124\u2126\u2128\u212a-\u212d\u212f-\u2139\u213c-\u213f\u2145-\u2149\u214e\u2160-\u2188\u2c00-\u2c2e\u2c30-\u2c5e\u2c60-\u2ce4\u2ceb-\u2cee\u2cf2\u2cf3\u2d00-\u2d25\u2d27\u2d2d\u2d30-\u2d67\u2d6f\u2d80-\u2d96\u2da0-\u2da6\u2da8-\u2dae\u2db0-\u2db6\u2db8-\u2dbe\u2dc0-\u2dc6\u2dc8-\u2dce\u2dd0-\u2dd6\u2dd8-\u2dde\u2e2f\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303c\u3041-\u3096\u309d-\u309f\u30a1-\u30fa\u30fc-\u30ff\u3105-\u312d\u3131-\u318e\u31a0-\u31ba\u31f0-\u31ff\u3400-\u4db5\u4e00-\u9fcc\ua000-\ua48c\ua4d0-\ua4fd\ua500-\ua60c\ua610-\ua61f\ua62a\ua62b\ua640-\ua66e\ua67f-\ua697\ua6a0-\ua6ef\ua717-\ua71f\ua722-\ua788\ua78b-\ua78e\ua790-\ua793\ua7a0-\ua7aa\ua7f8-\ua801\ua803-\ua805\ua807-\ua80a\ua80c-\ua822\ua840-\ua873\ua882-\ua8b3\ua8f2-\ua8f7\ua8fb\ua90a-\ua925\ua930-\ua946\ua960-\ua97c\ua984-\ua9b2\ua9cf\uaa00-\uaa28\uaa40-\uaa42\uaa44-\uaa4b\uaa60-\uaa76\uaa7a\uaa80-\uaaaf\uaab1\uaab5\uaab6\uaab9-\uaabd\uaac0\uaac2\uaadb-\uaadd\uaae0-\uaaea\uaaf2-\uaaf4\uab01-\uab06\uab09-\uab0e\uab11-\uab16\uab20-\uab26\uab28-\uab2e\uabc0-\uabe2\uac00-\ud7a3\ud7b0-\ud7c6\ud7cb-\ud7fb\uf900-\ufa6d\ufa70-\ufad9\ufb00-\ufb06\ufb13-\ufb17\ufb1d\ufb1f-\ufb28\ufb2a-\ufb36\ufb38-\ufb3c\ufb3e\ufb40\ufb41\ufb43\ufb44\ufb46-\ufbb1\ufbd3-\ufd3d\ufd50-\ufd8f\ufd92-\ufdc7\ufdf0-\ufdfb\ufe70-\ufe74\ufe76-\ufefc\uff21-\uff3a\uff41-\uff5a\uff66-\uffbe\uffc2-\uffc7\uffca-\uffcf\uffd2-\uffd7\uffda-\uffdc][$A-Z\_a-z\xaa\xb5\xba\xc0-\xd6\xd8-\xf6\xf8-\u02c1\u02c6-\u02d1\u02e0-\u02e4\u02ec\u02ee\u0370-\u0374\u0376\u0377\u037a-\u037d\u0386\u0388-\u038a\u038c\u038e-\u03a1\u03a3-\u03f5\u03f7-\u0481\u048a-\u0527\u0531-\u0556\u0559\u0561-\u0587\u05d0-\u05ea\u05f0-\u05f2\u0620-\u064a\u066e\u066f\u0671-\u06d3\u06d5\u06e5\u06e6\u06ee\u06ef\u06fa-\u06fc\u06ff\u0710\u0712-\u072f\u074d-\u07a5\u07b1\u07ca-\u07ea\u07f4\u07f5\u07fa\u0800-\u0815\u081a\u0824\u0828\u0840-\u0858\u08a0\u08a2-\u08ac\u0904-\u0939\u093d\u0950\u0958-\u0961\u0971-\u0977\u0979-\u097f\u0985-\u098c\u098f\u0990\u0993-\u09a8\u09aa-\u09b0\u09b2\u09b6-\u09b9\u09bd\u09ce\u09dc\u09dd\u09df-\u09e1\u09f0\u09f1\u0a05-\u0a0a\u0a0f\u0a10\u0a13-\u0a28\u0a2a-\u0a30\u0a32\u0a33\u0a35\u0a36\u0a38\u0a39\u0a59-\u0a5c\u0a5e\u0a72-\u0a74\u0a85-\u0a8d\u0a8f-\u0a91\u0a93-\u0aa8\u0aaa-\u0ab0\u0ab2\u0ab3\u0ab5-\u0ab9\u0abd\u0ad0\u0ae0\u0ae1\u0b05-\u0b0c\u0b0f\u0b10\u0b13-\u0b28\u0b2a-\u0b30\u0b32\u0b33\u0b35-\u0b39\u0b3d\u0b5c\u0b5d\u0b5f-\u0b61\u0b71\u0b83\u0b85-\u0b8a\u0b8e-\u0b90\u0b92-\u0b95\u0b99\u0b9a\u0b9c\u0b9e\u0b9f\u0ba3\u0ba4\u0ba8-\u0baa\u0bae-\u0bb9\u0bd0\u0c05-\u0c0c\u0c0e-\u0c10\u0c12-\u0c28\u0c2a-\u0c33\u0c35-\u0c39\u0c3d\u0c58\u0c59\u0c60\u0c61\u0c85-\u0c8c\u0c8e-\u0c90\u0c92-\u0ca8\u0caa-\u0cb3\u0cb5-\u0cb9\u0cbd\u0cde\u0ce0\u0ce1\u0cf1\u0cf2\u0d05-\u0d0c\u0d0e-\u0d10\u0d12-\u0d3a\u0d3d\u0d4e\u0d60\u0d61\u0d7a-\u0d7f\u0d85-\u0d96\u0d9a-\u0db1\u0db3-\u0dbb\u0dbd\u0dc0-\u0dc6\u0e01-\u0e30\u0e32\u0e33\u0e40-\u0e46\u0e81\u0e82\u0e84\u0e87\u0e88\u0e8a\u0e8d\u0e94-\u0e97\u0e99-\u0e9f\u0ea1-\u0ea3\u0ea5\u0ea7\u0eaa\u0eab\u0ead-\u0eb0\u0eb2\u0eb3\u0ebd\u0ec0-\u0ec4\u0ec6\u0edc-\u0edf\u0f00\u0f40-\u0f47\u0f49-\u0f6c\u0f88-\u0f8c\u1000-\u102a\u103f\u1050-\u1055\u105a-\u105d\u1061\u1065\u1066\u106e-\u1070\u1075-\u1081\u108e\u10a0-\u10c5\u10c7\u10cd\u10d0-\u10fa\u10fc-\u1248\u124a-\u124d\u1250-\u1256\u1258\u125a-\u125d\u1260-\u1288\u128a-\u128d\u1290-\u12b0\u12b2-\u12b5\u12b8-\u12be\u12c0\u12c2-\u12c5\u12c8-\u12d6\u12d8-\u1310\u1312-\u1315\u1318-\u135a\u1380-\u138f\u13a0-\u13f4\u1401-\u166c\u166f-\u167f\u1681-\u169a\u16a0-\u16ea\u16ee-\u16f0\u1700-\u170c\u170e-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176c\u176e-\u1770\u1780-\u17b3\u17d7\u17dc\u1820-\u1877\u1880-\u18a8\u18aa\u18b0-\u18f5\u1900-\u191c\u1950-\u196d\u1970-\u1974\u1980-\u19ab\u19c1-\u19c7\u1a00-\u1a16\u1a20-\u1a54\u1aa7\u1b05-\u1b33\u1b45-\u1b4b\u1b83-\u1ba0\u1bae\u1baf\u1bba-\u1be5\u1c00-\u1c23\u1c4d-\u1c4f\u1c5a-\u1c7d\u1ce9-\u1cec\u1cee-\u1cf1\u1cf5\u1cf6\u1d00-\u1dbf\u1e00-\u1f15\u1f18-\u1f1d\u1f20-\u1f45\u1f48-\u1f4d\u1f50-\u1f57\u1f59\u1f5b\u1f5d\u1f5f-\u1f7d\u1f80-\u1fb4\u1fb6-\u1fbc\u1fbe\u1fc2-\u1fc4\u1fc6-\u1fcc\u1fd0-\u1fd3\u1fd6-\u1fdb\u1fe0-\u1fec\u1ff2-\u1ff4\u1ff6-\u1ffc\u2071\u207f\u2090-\u209c\u2102\u2107\u210a-\u2113\u2115\u2119-\u211d\u2124\u2126\u2128\u212a-\u212d\u212f-\u2139\u213c-\u213f\u2145-\u2149\u214e\u2160-\u2188\u2c00-\u2c2e\u2c30-\u2c5e\u2c60-\u2ce4\u2ceb-\u2cee\u2cf2\u2cf3\u2d00-\u2d25\u2d27\u2d2d\u2d30-\u2d67\u2d6f\u2d80-\u2d96\u2da0-\u2da6\u2da8-\u2dae\u2db0-\u2db6\u2db8-\u2dbe\u2dc0-\u2dc6\u2dc8-\u2dce\u2dd0-\u2dd6\u2dd8-\u2dde\u2e2f\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303c\u3041-\u3096\u309d-\u309f\u30a1-\u30fa\u30fc-\u30ff\u3105-\u312d\u3131-\u318e\u31a0-\u31ba\u31f0-\u31ff\u3400-\u4db5\u4e00-\u9fcc\ua000-\ua48c\ua4d0-\ua4fd\ua500-\ua60c\ua610-\ua61f\ua62a\ua62b\ua640-\ua66e\ua67f-\ua697\ua6a0-\ua6ef\ua717-\ua71f\ua722-\ua788\ua78b-\ua78e\ua790-\ua793\ua7a0-\ua7aa\ua7f8-\ua801\ua803-\ua805\ua807-\ua80a\ua80c-\ua822\ua840-\ua873\ua882-\ua8b3\ua8f2-\ua8f7\ua8fb\ua90a-\ua925\ua930-\ua946\ua960-\ua97c\ua984-\ua9b2\ua9cf\uaa00-\uaa28\uaa40-\uaa42\uaa44-\uaa4b\uaa60-\uaa76\uaa7a\uaa80-\uaaaf\uaab1\uaab5\uaab6\uaab9-\uaabd\uaac0\uaac2\uaadb-\uaadd\uaae0-\uaaea\uaaf2-\uaaf4\uab01-\uab06\uab09-\uab0e\uab11-\uab16\uab20-\uab26\uab28-\uab2e\uabc0-\uabe2\uac00-\ud7a3\ud7b0-\ud7c6\ud7cb-\ud7fb\uf900-\ufa6d\ufa70-\ufad9\ufb00-\ufb06\ufb13-\ufb17\ufb1d\ufb1f-\ufb28\ufb2a-\ufb36\ufb38-\ufb3c\ufb3e\ufb40\ufb41\ufb43\ufb44\ufb46-\ufbb1\ufbd3-\ufd3d\ufd50-\ufd8f\ufd92-\ufdc7\ufdf0-\ufdfb\ufe70-\ufe74\ufe76-\ufefc\uff21-\uff3a\uff41-\uff5a\uff66-\uffbe\uffc2-\uffc7\uffca-\uffcf\uffd2-\uffd7\uffda-\uffdc0-9\u0300-\u036f\u0483-\u0487\u0591-\u05bd\u05bf\u05c1\u05c2\u05c4\u05c5\u05c7\u0610-\u061a\u064b-\u0669\u0670\u06d6-\u06dc\u06df-\u06e4\u06e7\u06e8\u06ea-\u06ed\u06f0-\u06f9\u0711\u0730-\u074a\u07a6-\u07b0\u07c0-\u07c9\u07eb-\u07f3\u0816-\u0819\u081b-\u0823\u0825-\u0827\u0829-\u082d\u0859-\u085b\u08e4-\u08fe\u0900-\u0903\u093a-\u093c\u093e-\u094f\u0951-\u0957\u0962\u0963\u0966-\u096f\u0981-\u0983\u09bc\u09be-\u09c4\u09c7\u09c8\u09cb-\u09cd\u09d7\u09e2\u09e3\u09e6-\u09ef\u0a01-\u0a03\u0a3c\u0a3e-\u0a42\u0a47\u0a48\u0a4b-\u0a4d\u0a51\u0a66-\u0a71\u0a75\u0a81-\u0a83\u0abc\u0abe-\u0ac5\u0ac7-\u0ac9\u0acb-\u0acd\u0ae2\u0ae3\u0ae6-\u0aef\u0b01-\u0b03\u0b3c\u0b3e-\u0b44\u0b47\u0b48\u0b4b-\u0b4d\u0b56\u0b57\u0b62\u0b63\u0b66-\u0b6f\u0b82\u0bbe-\u0bc2\u0bc6-\u0bc8\u0bca-\u0bcd\u0bd7\u0be6-\u0bef\u0c01-\u0c03\u0c3e-\u0c44\u0c46-\u0c48\u0c4a-\u0c4d\u0c55\u0c56\u0c62\u0c63\u0c66-\u0c6f\u0c82\u0c83\u0cbc\u0cbe-\u0cc4\u0cc6-\u0cc8\u0cca-\u0ccd\u0cd5\u0cd6\u0ce2\u0ce3\u0ce6-\u0cef\u0d02\u0d03\u0d3e-\u0d44\u0d46-\u0d48\u0d4a-\u0d4d\u0d57\u0d62\u0d63\u0d66-\u0d6f\u0d82\u0d83\u0dca\u0dcf-\u0dd4\u0dd6\u0dd8-\u0ddf\u0df2\u0df3\u0e31\u0e34-\u0e3a\u0e47-\u0e4e\u0e50-\u0e59\u0eb1\u0eb4-\u0eb9\u0ebb\u0ebc\u0ec8-\u0ecd\u0ed0-\u0ed9\u0f18\u0f19\u0f20-\u0f29\u0f35\u0f37\u0f39\u0f3e\u0f3f\u0f71-\u0f84\u0f86\u0f87\u0f8d-\u0f97\u0f99-\u0fbc\u0fc6\u102b-\u103e\u1040-\u1049\u1056-\u1059\u105e-\u1060\u1062-\u1064\u1067-\u106d\u1071-\u1074\u1082-\u108d\u108f-\u109d\u135d-\u135f\u1712-\u1714\u1732-\u1734\u1752\u1753\u1772\u1773\u17b4-\u17d3\u17dd\u17e0-\u17e9\u180b-\u180d\u1810-\u1819\u18a9\u1920-\u192b\u1930-\u193b\u1946-\u194f\u19b0-\u19c0\u19c8\u19c9\u19d0-\u19d9\u1a17-\u1a1b\u1a55-\u1a5e\u1a60-\u1a7c\u1a7f-\u1a89\u1a90-\u1a99\u1b00-\u1b04\u1b34-\u1b44\u1b50-\u1b59\u1b6b-\u1b73\u1b80-\u1b82\u1ba1-\u1bad\u1bb0-\u1bb9\u1be6-\u1bf3\u1c24-\u1c37\u1c40-\u1c49\u1c50-\u1c59\u1cd0-\u1cd2\u1cd4-\u1ce8\u1ced\u1cf2-\u1cf4\u1dc0-\u1de6\u1dfc-\u1dff\u200c\u200d\u203f\u2040\u2054\u20d0-\u20dc\u20e1\u20e5-\u20f0\u2cef-\u2cf1\u2d7f\u2de0-\u2dff\u302a-\u302f\u3099\u309a\ua620-\ua629\ua66f\ua674-\ua67d\ua69f\ua6f0\ua6f1\ua802\ua806\ua80b\ua823-\ua827\ua880\ua881\ua8b4-\ua8c4\ua8d0-\ua8d9\ua8e0-\ua8f1\ua900-\ua909\ua926-\ua92d\ua947-\ua953\ua980-\ua983\ua9b3-\ua9c0\ua9d0-\ua9d9\uaa29-\uaa36\uaa43\uaa4c\uaa4d\uaa50-\uaa59\uaa7b\uaab0\uaab2-\uaab4\uaab7\uaab8\uaabe\uaabf\uaac1\uaaeb-\uaaef\uaaf5\uaaf6\uabe3-\uabea\uabec\uabed\uabf0-\uabf9\ufb1e\ufe00-\ufe0f\ufe20-\ufe26\ufe33\ufe34\ufe4d-\ufe4f\uff10-\uff19\uff3f]*$/.test(name);
}
exports.isValidIdentifier = isValidIdentifier;
function dotnetPropertyName(name, enclosingType) {
    if (name == enclosingType) {
        return '_' + name;
    }
    return isValidIdentifier(name) ? name : pascalCase(name);
}
exports.dotnetPropertyName = dotnetPropertyName;
function componentRefName(name) {
    return isValidIdentifier(lodash_1.camelCase(name))
        ? lodash_1.camelCase(name)
        : '_' + lodash_1.camelCase(name);
}
exports.componentRefName = componentRefName;
function propertyName(name) {
    return name.split(' ').join('');
}
exports.propertyName = propertyName;
function unwrapType(value) {
    return value.split('.').pop();
}
exports.unwrapType = unwrapType;
function indent(value, count) {
    if (count === void 0) { count = 4; }
    if (typeof value === 'number') {
        return ' '.repeat(value);
    }
    else if (typeof value === 'string') {
        return value.trim().replace(/^/gm, ' '.repeat(count));
    }
}
exports.indent = indent;
function variable(value) {
    var result = lodash_1.camelCase(value);
    return isValidIdentifier(result) ? result : "_" + result;
}
exports.variable = variable;
function comma(index, countOrArray) {
    return appendIfNotLast(',', index, countOrArray);
}
exports.comma = comma;
function primitive(type) {
    switch (type) {
        case 'number':
        case 'integer':
        case 'string':
        case 'boolean':
            return true;
        default:
            return false;
    }
}
exports.primitive = primitive;
function emitActions(actions, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces) {
    if (spaces === void 0) { spaces = 12; }
    return actions
        .map(function (action) {
        return emitAction(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces);
    })
        .join('\n\n');
}
exports.emitActions = emitActions;
function emitAction(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces) {
    if (Event_1.isSet(action)) {
        return emitSet(action, pageName, componentName, eventName, properties, spaces);
    }
    else if (Event_1.isInvokeMethod(action)) {
        return emitInvokeMethod(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces);
    }
    else if (Event_1.isInvoke(action) || Event_1.isExport(action)) {
        return emitInvoke(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces);
    }
    else if (Event_1.isNavigate(action)) {
        return emitNavigate(action, pageName, componentName, eventName, properties, spaces);
    }
    else if (Event_1.isNavigateBack(action)) {
        return emitNavigateBack(action, pageName, componentName, eventName, properties, spaces);
    }
    else if (Event_1.isPopup(action)) {
        return emitPopup(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces);
    }
    else if (Event_1.isNotify(action)) {
        return emitNotify(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces);
    }
    else if (Event_1.isExecute(action)) {
        return emitExecute(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces);
    }
    else if (Event_1.isClosePopup(action)) {
        return emitClosePopup(action, pageName, componentName, eventName, properties, spaces);
    }
}
exports.emitAction = emitAction;
function generateUniqueVariable(variableName, variables) {
    var index = 0;
    while (variables.indexOf(variableName) >= 0) {
        variableName = "" + variableName + index;
        index++;
    }
    variables.push(variableName);
    return variableName;
}
function emitExecute(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces) {
    if (action.error && action.error.length > 0) {
        spaces += 4;
    }
    var indentation = indent(spaces);
    var blockIndent = indent(spaces + 4);
    var condition = action.condition && action.condition.trim();
    var variableName = generateUniqueVariable("executeResult", variables);
    var code = interpolateAsCode(pageName, componentName, eventName, properties, action.code);
    code = interpolateAsCSharp(code).trim();
    if (!code.endsWith(';')) {
        code += ';';
    }
    if ((action.then && action.then.length) ||
        (action.error && action.error.length)) {
        var then = emitActions(action.then || [], pageName, componentName, eventName, pages, dataSources, properties, variables, spaces + (condition ? 4 : 0)).replace(/\b(result)\b/g, variableName);
        var hasAwait = /await\s+/.test(code);
        code = condition
            ? [
                indentation + "if (" + interpolateAsCode(pageName, componentName, eventName, properties, condition) + ") {",
                indentation + "var " + variableName + " = await Task.Run(" + (hasAwait ? 'async ' : '') + "() =>",
                indentation + "{",
                "" + blockIndent + code,
                indentation + "});",
                then
            ].join('\n')
            : [
                indentation + "var " + variableName + " = await Task.Run(" + (hasAwait ? 'async ' : '') + "() =>",
                indentation + "{",
                "" + blockIndent + code,
                indentation + "});",
                then
            ].join('\n');
    }
    else {
        code = condition
            ? [
                indentation + "if (" + interpolateAsCode(pageName, componentName, eventName, properties, condition) + ") {",
                "" + blockIndent + code,
                "}"
            ].join('\n')
            : "" + indentation + code;
    }
    return wrapInTryCatch(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces, code);
}
function principalKeyName(schemas, fk, pkName) {
    var parentSchema = schemas.find(function (s) { return s.name == fk.parentEntityType; });
    if (parentSchema) {
        var pkProp = parentSchema.properties.find(function (p) { return p['x-original-name'] == pkName; });
        if (pkProp) {
            return pkProp.name;
        }
    }
    return pkName;
}
exports.principalKeyName = principalKeyName;
function severityToEnum(severity) {
    if (severity == 'info') {
        return 'NotificationSeverity.Info';
    }
    else if (severity == 'warn') {
        return 'NotificationSeverity.Warning';
    }
    else if (severity == 'error') {
        return 'NotificationSeverity.Error';
    }
    else if (severity == 'success') {
        return 'NotificationSeverity.Success';
    }
}
function emitNotify(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces) {
    if (!action.severity) {
        return;
    }
    var severity = severityToEnum(action.severity);
    var summary = interpolateAsCode(pageName, componentName, eventName, properties, interpolateAsCSharp(action.summary), true);
    var detail = interpolateAsCode(pageName, componentName, eventName, properties, interpolateAsCSharp(action.detail), true);
    var indentation = indent(spaces);
    var condition = action.condition && action.condition.trim();
    var blockIndent = indent(spaces + 4);
    var result = [];
    if (condition) {
        result.push(indentation + "if (" + interpolateAsCode(pageName, componentName, eventName, properties, condition) + ")", indentation + "{");
    }
    if (action.duration) {
        result.push(blockIndent + "NotificationService.Notify(" + severity + ", " + summary + ", " + detail + ", " + action.duration + ")" + ';');
    }
    else {
        result.push(blockIndent + "NotificationService.Notify(" + severity + ", " + summary + ", " + detail + ")" + ';');
    }
    if (action.then) {
        var then = emitActions(action.then || [], pageName, componentName, eventName, pages, dataSources, properties, variables, spaces + (condition ? 4 : 2));
        result.push(then);
    }
    if (condition) {
        result.push(indentation + "}");
    }
    return result.join('\n');
}
function emitNavigate(action, pageName, componentName, eventName, properties, spaces) {
    return emitActionWithCondition(action.condition, pageName, componentName, eventName, properties, spaces, function (blockIndent, result) {
        var indentation = indent(spaces);
        var parameters = params(action, pageName, componentName, eventName, properties, spaces);
        result.push(indentation + "UriHelper.NavigateTo(" + parameters + ");");
    });
}
function emitActionWithCondition(condition, pageName, componentName, eventName, properties, spaces, callback) {
    var indentation = indent(spaces);
    condition = condition && condition.trim();
    var blockIndent = "" + indentation + (condition ? '  ' : '');
    var result = [];
    if (condition) {
        result.push(indentation + "if (" + interpolateAsCode(pageName, componentName, eventName, properties, condition) + ") {");
    }
    callback(blockIndent, result);
    if (condition) {
        result.push(indentation + "}");
    }
    return result.join('\n');
}
function emitNavigateBack(action, pageName, componentName, eventName, properties, spaces) {
    return emitActionWithCondition(action.condition, pageName, componentName, eventName, properties, spaces, function (blockIndent, result) {
        result.push(blockIndent + "DialogService.Close();");
        result.push(blockIndent + "await JSRuntime.InvokeAsync<string>(\"window.history.back\");");
    });
}
function emitClosePopup(action, pageName, componentName, eventName, properties, spaces) {
    return emitActionWithCondition(action.condition, pageName, componentName, eventName, properties, spaces, function (blockIndent, result) {
        var dialogResult = action.result
            ? interpolateAsCode(pageName, componentName, eventName, properties, action.result)
            : 'null';
        result.push(blockIndent + "DialogService.Close(" + dialogResult + ");");
    });
}
function emitPopup(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces) {
    return emitActionWithCondition(action.condition, pageName, componentName, eventName, properties, spaces, function (blockIndent, result) {
        var dp = dialogParams(action, pageName, componentName, eventName, pages, properties);
        var parametersString = dp.parameters.length
            ? ", new Dictionary<string, object>() { " + dp.parameters + " }"
            : ', null';
        var optionsString = dp.options.length
            ? ", new DialogOptions(){ " + dp.options + " }"
            : '';
        if (action.then && action.then.length > 0) {
            var then = emitActions(action.then, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces + (action.condition ? 4 : 2));
            result.push(blockIndent + "var result = await DialogService.OpenAsync<" + dp.page + ">(" + dp.title + parametersString + optionsString + ");");
            result.push("" + then);
        }
        else {
            result.push(blockIndent + "DialogService.Open<" + dp.page + ">(" + dp.title + parametersString + optionsString + ");");
        }
    });
}
function throwCodegenError(pageName, componentName, eventName, message) {
    if (pageName && componentName && eventName) {
        throw new Error("Radzen: " + message + ".\nIn page: \"" + pageName + "\", component: " + componentName + ", event: " + (eventName
            .charAt(0)
            .toUpperCase() + eventName.substring(1)));
    }
    else {
        throw new Error("Radzen: " + message);
    }
}
exports.throwCodegenError = throwCodegenError;
function dialogParams(action, pageName, componentName, eventName, pages, properties) {
    var page = pages.find(function (page) { return kebabCase(page.name) == action.path; });
    if (!page) {
        throwCodegenError(pageName, componentName, eventName, "Cannot find page with path \"" + action.path + "\"");
    }
    var parameters = (action.parameters || [])
        .map(function (parameter) {
        return "{\"" + parameter.name + "\", " + interpolateAsCode(pageName, componentName, eventName, properties, interpolateAsCSharp(parameter.value)) + "}";
    })
        .join(', ');
    var options = [];
    ['top', 'left', 'width', 'height'].forEach(function (param) {
        if (typeof action[param] == 'number' || action[param]) {
            var value = !isNaN(action[param])
                ? action[param]
                : interpolateAsCode(pageName, componentName, eventName, properties, action[param]);
            if (!isNaN(value)) {
                options.push(lodash_1.upperFirst(param) + " = $\"{" + value + "}px\"");
            }
            else {
                options.push(lodash_1.upperFirst(param) + " = $\"{" + value + "}\"");
            }
        }
    });
    var title = action.title
        ? "" + interpolateAsCode(pageName, componentName, eventName, properties, action.title, true)
        : "\"" + page.name + "\"";
    return {
        title: title,
        page: "" + type(page.name),
        parameters: parameters,
        options: options
    };
}
function params(action, pageName, componentName, eventName, properties, spaces) {
    var path = interpolateAsCode(pageName, componentName, eventName, properties, action.path);
    var parameters = (action.parameters || []).map(function (parameter) {
        var value = interpolateAsCode(pageName, componentName, eventName, properties, parameter.value);
        return interpolateAsCSharp(value);
    });
    if (parameters.length) {
        return "$\"" + path + "/" + parameters.map(function (p) { return '{' + p + '}'; }).join('/') + "\"";
    }
    else {
        return "\"" + path + "\"";
    }
}
function interpolate(value, pageName, componentName, eventName) {
    var _a = parseExpression(value, pageName, componentName, eventName), strings = _a.strings, values = _a.values;
    return strings
        .map(function (str, index) {
        if (strings.length <= index) {
            return str;
        }
        var value = values[index - 1];
        return value ? value + str : str;
    })
        .join('');
}
exports.interpolate = interpolate;
function interpolateFilterOperationAsCode(operation) {
    var operations = {
        eq: '==',
        ne: '!=',
        lt: '<',
        le: '<=',
        gt: '>',
        ge: '>=',
        in: 'Contains'
    };
    return operations[operation];
}
exports.interpolateFilterOperationAsCode = interpolateFilterOperationAsCode;
function interpolateAsCodeForFilter(pageName, componentName, eventName, properties, expression, quote) {
    if (expression === void 0) { expression = ''; }
    if (quote === void 0) { quote = false; }
    var _a = parseExpression(expression, pageName, componentName, eventName), strings = _a.strings, values = _a.values;
    var value = strings
        .map(function (str, index) {
        if (strings.length <= index) {
            return str;
        }
        var value = values[index - 1];
        if (value) {
            var _a = value.split('.'), name_1 = _a[0], rest = _a.slice(1);
            if (properties.indexOf(name_1) >= 0) {
                name_1 = "" + name_1;
            }
            var expression_1 = [name_1].concat(rest).join('.');
            value = quote ? '{' + expression_1 + '}' : expression_1;
        }
        return value ? value + str : str;
    })
        .join('')
        .replace(/[\""]/g, '""')
        .replace(/[\'']/g, '""');
    return value;
}
exports.interpolateAsCodeForFilter = interpolateAsCodeForFilter;
function interpolateAsCode(pageName, componentName, eventName, properties, expression, quote) {
    if (expression === void 0) { expression = ''; }
    if (quote === void 0) { quote = false; }
    var _a = parseExpression(expression, pageName, componentName, eventName), strings = _a.strings, values = _a.values;
    var value = strings
        .map(function (str, index) {
        if (strings.length <= index) {
            return str;
        }
        var value = values[index - 1];
        if (value) {
            var _a = value.split('.'), name_2 = _a[0], rest = _a.slice(1);
            if (properties.indexOf(name_2) >= 0) {
                name_2 = "" + name_2;
            }
            var expression_2 = [name_2].concat(rest).join('.');
            value = quote ? '{' + expression_2 + '}' : expression_2;
        }
        return value ? value + str : str;
    })
        .join('')
        .replace(/'/g, '"');
    return quote ? '$"' + value + '"' : value;
}
exports.interpolateAsCode = interpolateAsCode;
function interpolateAsBlazorAttribute(properties, expression) {
    if (expression === void 0) { expression = ''; }
    return expression.replace(BOUND_EXPRESSION, function ($0, $1) {
        return /[!@#$%^&*()_+\-=\[\]{};':"\\|,.<>\/?]+/.test($1) ? "@(" + $1 + ")" : "@" + $1;
    });
}
exports.interpolateAsBlazorAttribute = interpolateAsBlazorAttribute;
function interpolateAsCSharp(value) {
    return value == '' || value == null
        ? 'null'
        : value
            .replace(/parameters./g, '')
            .replace(/Invoke\(/g, 'InvokeAsync(')
            .replace(/event.username/g, 'args.Username')
            .replace(/event.password/g, 'args.Password')
            .replace(/\b(event)\b/g, 'args')
            .replace(/args.skip/g, 'args.Skip')
            .replace(/args.value/g, 'args.Value')
            .replace(/args.top/g, 'args.Top')
            .replace(/args.orderby/g, 'args.OrderBy')
            .replace(/args.filter/g, 'args.Filter')
            .replace(/result.value/g, 'result');
}
function emitInvokeMethod(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces) {
    if (!action.name) {
        return;
    }
    if (action.error && action.error.length > 0) {
        spaces += 4;
    }
    var indentation = indent(spaces);
    var blockIndent = indent(spaces + 4);
    var args = action.parameters || [];
    var parameters = args.map(function (parameter) {
        var value = interpolateAsCode(pageName, componentName, eventName, properties, interpolateAsCSharp(parameter.value), parameter.type == 'string');
        return value;
    });
    var resultName = generateUniqueVariable(lodash_1.camelCase(action.name) + "Result", variables);
    var isAsync = action.schema && action.schema.async;
    var invoke = "" + (isAsync ? 'await ' : '') + action.name + "(" + parameters + ");";
    if (action.schema &&
        action.schema.dotnetType != 'System.Void' &&
        action.schema.dotnetType != 'void') {
        invoke = "var " + resultName + " = " + invoke;
    }
    var condition = action.condition && action.condition.trim();
    var then = emitActions(action.then || [], pageName, componentName, eventName, pages, dataSources, properties, variables, spaces).replace(/\b(result)\b/g, resultName);
    var lines = condition
        ? [
            indentation + "if (" + interpolateAsCode(pageName, componentName, eventName, properties, condition) + ")",
            indentation + "{",
            "" + blockIndent + invoke,
            "" + then,
            indentation + "}"
        ]
        : ["" + indentation + invoke, "" + then];
    var code = joinNonEmptyLines(lines);
    return wrapInTryCatch(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces, code);
}
function joinNonEmptyLines(lines) {
    return lines.filter(function (line) { return line.length > 0; }).join('\n');
}
function wrapInTryCatch(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces, code) {
    var exceptionName = Event_1.isExecute(action)
        ? 'exception'
        : lodash_1.camelCase(action.name) + "Exception";
    exceptionName = generateUniqueVariable(exceptionName, variables);
    var error = emitActions(action.error || [], pageName, componentName, eventName, pages, dataSources, properties, variables, spaces).replace(/\b(result)\b/g, exceptionName);
    if (error) {
        // Remove Angular error handling code
        error = error.replace(/\s+canEdit = \!\(\w+\.status == 400\);\n*/m, '');
        error = error.replace(/\.error\.message/g, '.Message');
        var tryIndentation = indent(spaces - 4);
        code = [
            tryIndentation + "try",
            tryIndentation + "{",
            code,
            tryIndentation + "}",
            tryIndentation + "catch (Exception " + exceptionName + ")",
            tryIndentation + "{",
            error,
            tryIndentation + "}"
        ].join('\n');
    }
    return code;
}
function odataDataSourceParameter(parameter, action, pageName, componentName, eventName, properties) {
    var args = action.parameters || [];
    var allArgs = action.type == 'export'
        ? [{ name: '$format', value: action.format || 'csv' }].concat(args)
        : args;
    var argument = allArgs.find(function (argument) { return argument.name == parameter.name; }) || { value: '' };
    var value;
    if (parameter.name == '$filter' && Array.isArray(argument.value)) {
        var logicalOperator = argument.operator || 'and';
        var filterToString_1 = function (f) {
            if (f.filters) {
                return "(" + f.filters.map(filterToString_1).join(" " + f.operator + " ") + ")";
            }
            if (!f.property && f.value) {
                return interpolateAsCodeForFilter(pageName, componentName, eventName, properties, f.value, true);
            }
            var oDataProp = f.property.replace(/[.]/g, '/');
            var oDataValue = interpolateAsCodeForFilter(pageName, componentName, eventName, properties, f.value, true);
            var parsedDate = Date.parse(oDataValue);
            if (isNaN(oDataValue) &&
                !isNaN(parsedDate) &&
                f.type == 'string' &&
                (f.format == 'date-time' || f.format == 'date')) {
                var date = oDataValue ? new Date(oDataValue) : null;
                if (oDataValue) {
                    date =
                        f.format == 'date-time'
                            ? new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds()))
                            : new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate()));
                }
                oDataValue = date ? date.toISOString() : 'null';
            }
            if (f.operation == 'contains' ||
                f.operation == 'startswith' ||
                f.operation == 'endswith') {
                return f.operation + "(" + oDataProp + "," + oDataValue + ")";
            }
            if (f.operation == 'in' && oDataValue.indexOf('(') == -1) {
                return oDataProp + " " + f.operation + " (" + oDataValue + ")";
            }
            return oDataProp + " " + f.operation + " " + oDataValue;
        };
        value =
            '$"' +
                argument.value
                    .filter(function (f) { return f.property || f.value || f.filters; })
                    .map(filterToString_1)
                    .join(" " + logicalOperator + " ") +
                '"';
    }
    else if (parameter.name == '$orderby' && Array.isArray(argument.value)) {
        value =
            '$"' +
                argument.value
                    .filter(function (f) { return f.property || f.value || f.filters; })
                    .map(function (f) {
                    if (!f.property && f.value) {
                        return interpolateAsCodeForFilter(pageName, componentName, eventName, properties, f.value, true);
                    }
                    return f.property.replace(/[.]/g, '/') + " " + f.direction;
                })
                    .join(',') +
                '"';
    }
    else {
        value = interpolateAsCode(pageName, componentName, eventName, properties, argument.value, parameter.type == 'string');
    }
    if (parameter.in == 'path' &&
        ['string', 'number', 'integer', 'boolean'].indexOf(parameter.type) < 0) {
        return interpolateParameterAsCode(parameter, value);
    }
    return argument.value
        ? variable(parameter.name.replace('$', '')) + ":" + interpolateAsCSharp(value)
        : null;
}
function sqlDataSourceParameter(parameter, action, pageName, componentName, eventName, properties, dataSource, operation) {
    var args = action.parameters || [];
    var allArgs = action.type == 'export'
        ? [{ name: '$format', value: action.format || 'csv' }].concat(args)
        : args;
    var argument = allArgs.find(function (argument) { return argument.name == parameter.name; });
    if (!argument) {
        if (parameter.required) {
            throwCodegenError(pageName, componentName, eventName, "The \"" + dataSource.name + "." + operation.name + "\" operation is missing an argument for the required parameter \"" + parameter.name + "\"");
        }
        if (parameter.default == undefined) {
            return null;
        }
        return;
    }
    var value;
    if (parameter.name == '$filter' && Array.isArray(argument.value)) {
        var logicalOperator = (argument.operator || 'and')
            .replace('and', '&&')
            .replace('or', '||');
        var filterToString_2 = function (f) {
            if (f.filters) {
                return "(" + f.filters
                    .map(filterToString_2)
                    .join(" " + f.operator.replace('and', '&&').replace('or', '||') + " ") + ")";
            }
            if (!f.property && f.value) {
                return interpolateAsCodeForFilter(pageName, componentName, eventName, properties, f.value, true);
            }
            var oDataProp = f.property;
            var oDataValue = interpolateAsCodeForFilter(pageName, componentName, eventName, properties, f.value, true);
            if (f.type == 'string' &&
                (f.format == 'date-time' || f.format == 'date')) {
                var date = oDataValue ? new Date(oDataValue) : null;
                if (oDataValue) {
                    date =
                        f.format == 'date-time'
                            ? new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds()))
                            : new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate()));
                }
                oDataValue = date ? "DateTime.Parse(\"" + date.toISOString() + "\")" : 'null';
            }
            if (f.operation == 'contains' ||
                f.operation == 'startswith' ||
                f.operation == 'endswith') {
                return "i." + oDataProp + "." + pascalCase(f.operation) + "(" + oDataValue + ")";
            }
            if (f.operation == 'in' && oDataValue.indexOf('(') == -1) {
                return oDataValue + ".Contains(i." + oDataProp + ")";
            }
            return "i." + oDataProp + " " + interpolateFilterOperationAsCode(f.operation) + " " + oDataValue;
        };
        value =
            'Filter = $@"' +
                'i => ' +
                argument.value.map(filterToString_2).join(" " + logicalOperator + " ") +
                '"';
    }
    else if (parameter.name == '$orderby' && Array.isArray(argument.value)) {
        value =
            'OrderBy = $"' +
                argument.value
                    .map(function (f) {
                    if (!f.property && f.value) {
                        return interpolateAsCodeForFilter(pageName, componentName, eventName, properties, f.value, true);
                    }
                    return f.property.replace(/[.]/g, '/') + " " + f.direction;
                })
                    .join(',') +
                '"';
    }
    else if (parameter.name == '$expand') {
        value = "Expand = \"" + argument.value + "\"";
    }
    else if (parameter.name == '$skip') {
        value = "Skip = " + interpolateAsCodeForFilter(pageName, componentName, eventName, properties, argument.value);
    }
    else if (parameter.name == '$top') {
        value = "Top = " + interpolateAsCodeForFilter(pageName, componentName, eventName, properties, argument.value);
    }
    else if (parameter.name == '$filter' && !Array.isArray(argument.value)) {
        value = "Filter = $@\"" + interpolateAsCodeForFilter(pageName, componentName, eventName, properties, argument.value, true) + "\"";
    }
    else if (parameter.name == '$orderby' && !Array.isArray(argument.value)) {
        value = "OrderBy = $\"" + interpolateAsCodeForFilter(pageName, componentName, eventName, properties, argument.value, true) + "\"";
    }
    else if (parameter.name == '$format') {
        value = null;
    }
    else {
        value = interpolateAsCode(pageName, componentName, eventName, properties, argument.value, parameter.type == 'string' && !parameter.format);
    }
    if (parameter.in == 'path' &&
        value.includes('parameters.') &&
        ['string', 'number', 'integer', 'boolean'].indexOf(parameter.type) >= 0) {
        return interpolateParameterAsCode(parameter, value);
    }
    return value ? interpolateAsCSharp(value) : null;
}
function emitInvoke(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces) {
    if (!action.name) {
        return;
    }
    var _a = action.name.split('.'), target = _a[0], method = _a[1];
    var dataSource = dataSources.find(function (dataSource) { return dataSource.name == target; });
    if (!dataSource) {
        throwCodegenError(pageName, componentName, eventName, "Cannot find data source with name \"" + target + "\"");
    }
    var operation = lodash_1.flatten(lodash_1.map(dataSource.resources, function (operations) { return operations; })).find(function (operation) { return operation.name == method; });
    if (!operation) {
        throwCodegenError(pageName, componentName, eventName, "Cannot find operation with name \"" + method + "\" in the \"" + target + "\" data source");
    }
    if (properties.indexOf(lodash_1.camelCase(target)) >= 0) {
        target = "" + pascalCase(target);
    }
    if (action.error && action.error.length > 0) {
        spaces += 4;
    }
    var expression = [target, pascalCase(method)].join('.');
    var indentation = indent(spaces);
    var blockIndent = indent(spaces + 4);
    var parameters = lodash_1.orderBy(operation.parameters, ['default'], ['desc'])
        .map(function (parameter) {
        if (dataSource.connector || dataSource.name == 'Security') {
            return sqlDataSourceParameter(parameter, action, pageName, componentName, eventName, properties, dataSource, operation);
        }
        else if (!dataSource.connector && dataSource.type == 'odata') {
            return odataDataSourceParameter(parameter, action, pageName, componentName, eventName, properties);
        }
        else {
            var argument = (action.parameters || []).find(function (argument) { return argument.name == parameter.name; });
            return interpolateParameterAsCode(parameter, argument ? argument.value : null);
        }
    })
        .filter(function (value) { return !!value; })
        .join(', ');
    var invoke = '';
    var variableName = '';
    var query = dataSource.connector &&
        (parameters.includes('Filter = ') ||
            parameters.includes('OrderBy = ') ||
            parameters.includes('Expand = ') ||
            parameters.includes('Top = ') ||
            parameters.includes('Skip = '));
    if (action.type == 'export') {
        var exportMethodName = expression.replace('Get', 'Export') + "To" + (action.format == 'csv' || !action.format ? 'CSV' : 'Excel');
        invoke = query
            ? "await " + exportMethodName + "(new Query() { " + parameters + " });"
            : "await " + exportMethodName + "();";
    }
    else {
        variableName = lodash_1.camelCase(expression) + "Result";
        invoke = query
            ? "var " + variableName + " = await " + expression + "(new Query() { " + parameters + " });"
            : "var " + variableName + " = await " + expression + "(" + parameters + ");";
        if (variables.indexOf(variableName) >= 0) {
            invoke = query
                ? variableName + " = await " + expression + "(new Query() { " + parameters + " });"
                : variableName + " = await " + expression + "(" + parameters + ");";
        }
        else {
            variables.push(variableName);
        }
    }
    var condition = action.condition && action.condition.trim();
    var then = emitActions(action.then || [], pageName, componentName, eventName, pages, dataSources, properties, variables, spaces).replace(/\b(result)\b/g, variableName);
    var code = condition
        ? [
            indentation + "if (" + interpolateAsCode(pageName, componentName, eventName, properties, condition) + ")",
            indentation + "{",
            "" + blockIndent + invoke,
            "" + then,
            indentation + "}"
        ].join('\n')
        : ["" + indentation + invoke, "" + then].join('\n').replace(/\n$/, '');
    return wrapInTryCatch(action, pageName, componentName, eventName, pages, dataSources, properties, variables, spaces, code);
}
function interpolateParameterAsCode(parameter, value) {
    var newValue = (value || '').replace(/parameters./g, '');
    if (parameter.type == 'integer') {
        if (!parameter.format || parameter.format == 'byte') {
            return "int.Parse($\"{" + newValue + "}\")";
        }
        else if (parameter.format == 'int8') {
            return "byte.Parse($\"{" + newValue + "}\")";
        }
        else if (parameter.format == 'int16') {
            return "Int16.Parse($\"{" + newValue + "}\")";
        }
        else if (parameter.format == 'int64') {
            return "Int64.Parse($\"{" + newValue + "}\")";
        }
    }
    else if (parameter.type == 'boolean') {
        return "bool.Parse($\"{" + newValue + "}\")";
    }
    else if (parameter.type == 'number') {
        return (parameter.format || 'decimal') + ".Parse($\"{" + newValue + "}\")";
    }
    else if (parameter.type == 'string' && parameter.format == 'uuid') {
        return "Guid.Parse($\"{" + newValue + "}\")";
    }
    else if (parameter.type == 'string' && parameter.format == 'binary') {
        return "System.Text.Encoding.Default.GetBytes($\"{" + newValue + "}\")";
    }
    else if (parameter.type == 'string' && parameter.format == 'date-time') {
        return "DateTime.Parse($\"{" + newValue + "}\")";
    }
    return newValue;
}
exports.interpolateParameterAsCode = interpolateParameterAsCode;
function emitSet(action, pageName, componentName, eventName, properties, spaces) {
    if (!action.name) {
        return;
    }
    var value = interpolateAsCode(pageName, componentName, eventName, properties, action.value);
    value = interpolateAsCSharp(value);
    var indentation = indent(spaces);
    var blockIndent = indent(spaces + 4);
    var condition = action.condition && action.condition.trim();
    return condition
        ? [
            indentation + "if (" + interpolateAsCode(pageName, componentName, eventName, properties, condition) + ") {",
            "" + blockIndent + action.name + " = " + value + ";",
            indentation + "}"
        ].join('\n')
        : "" + indentation + action.name + " = " + value + ";";
}
var BOUND_EXPRESSION = /\${(.*?)\}/g;
function parseExpression(value, pageName, componentName, eventName) {
    try {
        var parseResult = esprima.parse('`' + value + '`');
        var expression = parseResult.body[0].expression;
        var strings = expression.quasis.map(function (str) {
            return str.value.raw;
        });
        var values = expression.expressions.map(function (exp) {
            var code = escodegen.generate(exp);
            return code;
        });
        return {
            strings: strings,
            values: values
        };
    }
    catch (err) {
        throwCodegenError(pageName, componentName, eventName, "Cannot generate code for \"" + value + "\": " + err.message.replace('ASSERT: ', ''));
    }
}
exports.parseExpression = parseExpression;
function isBoundExpression(expression) {
    return BOUND_EXPRESSION.test(expression);
}
exports.isBoundExpression = isBoundExpression;
function appendIfNotLast(str, index, countOrArray) {
    var count;
    if (Array.isArray(countOrArray)) {
        count = countOrArray.length - 1;
    }
    else {
        count = countOrArray;
    }
    return count > index ? str : '';
}
exports.appendIfNotLast = appendIfNotLast;
//# sourceMappingURL=env.js.map